@@ -136,9 +136,11 @@ static std::string getDirectoryBorderStyle(const DotDirProperty &property)
 
 static TextStream &common_attributes(TextStream &t, const DirDef *const dir, const DotDirProperty &prop)
 {
+  QCString url = dir->getOutputFileBase();
+  addHtmlExtensionIfMissing(url);
   return t <<
     "style=\""   << getDirectoryBorderStyle(prop) << "\", "
-    "URL=\""     << addHtmlExtensionIfMissing(dir->getOutputFileBase()) << "\","
+    "URL=\""     << url << "\","
     "tooltip=\"" << escapeTooltip(dir->briefDescriptionAsTooltip()) << "\"";
 }
 
@@ -218,7 +220,7 @@ static void addDependencies(DirRelations &dependencies,const DirDef *const srcDi
     if (!dstDir->isParentOf(srcDir) && (isLeaf || usedDirectory->hasDirectSrcDeps()))
     {
       QCString relationName;
-      relationName.sprintf("dir_%06d_%06d", srcDir->dirCount(), dstDir->dirCount());
+      relationName.sprintf("dir_%06d_%06d", srcDir->dirIndex(), dstDir->dirIndex());
       bool directRelation = isLeaf ? usedDirectory->hasDirectDstDeps() : usedDirectory->hasDirectDeps();
       auto &&dependency = std::make_unique<DirRelation>(relationName, srcDir, usedDirectory.get());
       auto &&pair = std::make_pair(std::move(dependency),directRelation);
@@ -387,7 +389,10 @@ void writeDotDirDepGraph(TextStream &t,const DirDef *dd,bool linkRelations)
         t << " [headlabel=\"" << nrefs << "\", labeldistance=1.5";
         if (linkRelations)
         {
-          t << " headhref=\"" << addHtmlExtensionIfMissing(relationName) << "\"";
+          QCString fn = relationName;
+          addHtmlExtensionIfMissing(fn);
+          t << " headhref=\"" << fn << "\"";
+          t << " href=\"" << fn << "\"";
         }
         t << " color=\"steelblue1\" fontcolor=\"steelblue1\"];\n";
       }
