@@ -13,8 +13,6 @@
  *
  */
 
-#include <qregexp.h>
-
 #include "pagedef.h"
 #include "groupdef.h"
 #include "docparser.h"
@@ -32,60 +30,61 @@
 class PageDefImpl : public DefinitionMixin<PageDef>
 {
   public:
-    PageDefImpl(const char *f,int l,const char *n,const char *d,const char *t);
-    virtual ~PageDefImpl();
+    PageDefImpl(const QCString &f,int l,const QCString &n,const QCString &d,const QCString &t);
+    ~PageDefImpl() override;
 
-    virtual void setFileName(const char *name);
-    virtual void setLocalToc(const LocalToc &tl);
-    virtual void setShowLineNo(bool);
-    virtual DefType definitionType() const { return TypePage; }
-    virtual bool isLinkableInProject() const { return /*hasDocumentation() &&*/ !isReference(); }
-    virtual bool isLinkable() const { return isLinkableInProject() || isReference(); }
-    virtual QCString getOutputFileBase() const;
-    virtual QCString anchor() const { return QCString(); }
-    virtual void findSectionsInDocumentation();
-    virtual QCString title() const { return m_title; }
-    virtual GroupDef *  getGroupDef() const;
-    virtual PageSDict * getSubPages() const { return m_subPageDict; }
-    virtual void addInnerCompound(const Definition *d);
-    virtual bool visibleInIndex() const;
-    virtual bool documentedPage() const;
-    virtual bool hasSubPages() const;
-    virtual bool hasParentPage() const;
-    virtual bool hasTitle() const;
-    virtual LocalToc localToc() const { return m_localToc; }
-    virtual void setPageScope(Definition *d){ m_pageScope = d; }
-    virtual Definition *getPageScope() const { return m_pageScope; }
-    virtual QCString displayName(bool=TRUE) const { return hasTitle() ? m_title : DefinitionMixin::name(); }
-    virtual bool showLineNo() const;
-    virtual void writeDocumentation(OutputList &ol);
-    virtual void writeTagFile(FTextStream &);
-    virtual void setNestingLevel(int l);
-    virtual void writePageDocumentation(OutputList &ol);
+    void setFileName(const QCString &name) override;
+    void setLocalToc(const LocalToc &tl) override;
+    void setShowLineNo(bool) override;
+    DefType definitionType() const override { return TypePage; }
+    CodeSymbolType codeSymbolType() const override { return CodeSymbolType::Default; }
+    bool isLinkableInProject() const override { return /*hasDocumentation() &&*/ !isReference(); }
+    bool isLinkable() const override { return isLinkableInProject() || isReference(); }
+    QCString getOutputFileBase() const override;
+    QCString anchor() const override { return QCString(); }
+    void findSectionsInDocumentation() override;
+    QCString title() const override { return m_title; }
+    const GroupDef * getGroupDef() const override;
+    const PageLinkedRefMap &getSubPages() const override { return m_subPages; }
+    void addInnerCompound(Definition *d) override;
+    bool visibleInIndex() const override;
+    bool documentedPage() const override;
+    bool hasSubPages() const override;
+    bool hasParentPage() const override;
+    bool hasTitle() const override;
+    LocalToc localToc() const override { return m_localToc; }
+    void setPageScope(Definition *d) override { m_pageScope = d; }
+    Definition *getPageScope() const override { return m_pageScope; }
+    QCString displayName(bool=TRUE) const override { return hasTitle() ? m_title : DefinitionMixin::name(); }
+    bool showLineNo() const override;
+    void setTitle(const QCString &title) override;
+    void writeDocumentation(OutputList &ol) override;
+    void writeTagFile(TextStream &) override;
+    void setNestingLevel(int l) override;
+    void writePageDocumentation(OutputList &ol) const override;
 
   private:
     QCString m_fileName;
     QCString m_title;
-    PageSDict *m_subPageDict;                 // list of pages in the group
+    PageLinkedRefMap m_subPages;                 // list of pages in the group
     Definition *m_pageScope;
     int m_nestingLevel;
     LocalToc m_localToc;
     bool m_showLineNo;
 };
 
-PageDef *createPageDef(const char *f,int l,const char *n,const char *d,const char *t)
+std::unique_ptr<PageDef> createPageDef(const QCString &f,int l,const QCString &n,const QCString &d,const QCString &t)
 {
-  return new PageDefImpl(f,l,n,d,t);
+  return std::make_unique<PageDefImpl>(f,l,n,d,t);
 }
 
 //------------------------------------------------------------------------------------------
 
-PageDefImpl::PageDefImpl(const char *f,int l,const char *n,
-                 const char *d,const char *t)
- : DefinitionMixin(f,l,1,n), m_title(t)
+PageDefImpl::PageDefImpl(const QCString &f,int l,const QCString &n,
+                 const QCString &d,const QCString &t)
+ : DefinitionMixin(f,l,1,n), m_title(!t.isEmpty() ? t : n)
 {
   setDocumentation(d,f,l);
-  m_subPageDict = new PageSDict(7);
   m_pageScope = 0;
   m_nestingLevel = 0;
   m_fileName = ::convertNameToFile(n,FALSE,TRUE);
@@ -94,7 +93,6 @@ PageDefImpl::PageDefImpl(const char *f,int l,const char *n,
 
 PageDefImpl::~PageDefImpl()
 {
-  delete m_subPageDict;
 }
 
 void PageDefImpl::findSectionsInDocumentation()
@@ -103,10 +101,9 @@ void PageDefImpl::findSectionsInDocumentation()
   docFindSections(documentation(),this,docFile());
 }
 
-GroupDef *PageDefImpl::getGroupDef() const
+const GroupDef *PageDefImpl::getGroupDef() const
 {
-  GroupList *groups = partOfGroups();
-  return groups!=0 ? groups->getFirst() : 0;
+  return !partOfGroups().empty() ? partOfGroups().front() : 0;
 }
 
 QCString PageDefImpl::getOutputFileBase() const
@@ -117,21 +114,21 @@ QCString PageDefImpl::getOutputFileBase() const
     return m_fileName;
 }
 
-void PageDefImpl::setFileName(const char *name)
+void PageDefImpl::setFileName(const QCString &name)
 {
   m_fileName = name;
 }
 
-void PageDefImpl::addInnerCompound(const Definition *def)
+void PageDefImpl::addInnerCompound(Definition *def)
 {
   if (def->definitionType()==Definition::TypePage)
   {
     PageDef *pd = const_cast<PageDef*>(toPageDef(def));
     if (pd)
     {
-      m_subPageDict->append(pd->name(),pd);
+      m_subPages.add(pd->name(),pd);
       pd->setOuterScope(this);
-      if (this==Doxygen::mainPage)
+      if (this==Doxygen::mainPage.get())
       {
         pd->setNestingLevel(m_nestingLevel);
       }
@@ -149,7 +146,7 @@ bool PageDefImpl::hasParentPage() const
          getOuterScope()->definitionType()==Definition::TypePage;
 }
 
-void PageDefImpl::writeTagFile(FTextStream &tagFile)
+void PageDefImpl::writeTagFile(TextStream &tagFile)
 {
   bool found = name()=="citelist";
   for (RefListManager::Ptr &rl : RefListManager::instance())
@@ -162,20 +159,35 @@ void PageDefImpl::writeTagFile(FTextStream &tagFile)
   }
   if (!found) // not one of the generated related pages
   {
-    tagFile << "  <compound kind=\"page\">" << endl;
-    tagFile << "    <name>" << name() << "</name>" << endl;
-    tagFile << "    <title>" << convertToXML(title()) << "</title>" << endl;
-    tagFile << "    <filename>" << convertToXML(getOutputFileBase())<< Doxygen::htmlFileExtension << "</filename>" << endl;
+    QCString fn = getOutputFileBase();
+    addHtmlExtensionIfMissing(fn);
+    tagFile << "  <compound kind=\"page\">\n";
+    tagFile << "    <name>" << name() << "</name>\n";
+    tagFile << "    <title>" << convertToXML(title()) << "</title>\n";
+    tagFile << "    <filename>" << fn << "</filename>\n";
+    for (const auto &subPage : m_subPages)
+    {
+      QCString sfn = subPage->getOutputFileBase();
+      addHtmlExtensionIfMissing(sfn);
+      tagFile << "    <subpage>" << sfn << "</subpage>\n";
+    }
     writeDocAnchorsToTagFile(tagFile);
-    tagFile << "  </compound>" << endl;
+    tagFile << "  </compound>\n";
   }
 }
 
 void PageDefImpl::writeDocumentation(OutputList &ol)
 {
-  static bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
+  bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
+  int hierarchyLevel = -1; // Pages start at the root
+  PageDef *pd = this;
+  while (pd->hasParentPage())
+  {
+    pd = (PageDef *)pd->getOuterScope();
+    ++hierarchyLevel;
+  }
 
-  //outputList->disable(OutputGenerator::Man);
+  //outputList->disable(OutputType::Man);
   QCString pageName,manPageName;
   pageName    = escapeCharsInString(name(),FALSE,TRUE);
   manPageName = escapeCharsInString(name(),TRUE,TRUE);
@@ -185,25 +197,15 @@ void PageDefImpl::writeDocumentation(OutputList &ol)
   ol.pushGeneratorState();
   //1.{
 
-  if (m_nestingLevel>0
-      //&& // a sub page
-      //(Doxygen::mainPage==0 || getOuterScope()!=Doxygen::mainPage) // and not a subpage of the mainpage
-     )
-  {
-    // do not generate sub page output for RTF and LaTeX, as these are
-    // part of their parent page
-    ol.disableAll();
-    ol.enable(OutputGenerator::Man);
-    ol.enable(OutputGenerator::Html);
-  }
-
   ol.pushGeneratorState();
   //2.{
-  ol.disableAllBut(OutputGenerator::Man);
-  startFile(ol,getOutputFileBase(),manPageName,title(),HLI_Pages,!generateTreeView);
+  ol.disableAllBut(OutputType::Man);
+  startFile(ol,getOutputFileBase(),manPageName,title(),HighlightedItem::Pages,!generateTreeView,
+            QCString() /* altSidebarName */, hierarchyLevel);
   ol.enableAll();
-  ol.disable(OutputGenerator::Man);
-  startFile(ol,getOutputFileBase(),pageName,title(),HLI_Pages,!generateTreeView);
+  ol.disable(OutputType::Man);
+  startFile(ol,getOutputFileBase(),pageName,title(),HighlightedItem::Pages,!generateTreeView,
+            QCString() /* altSidebarName */, hierarchyLevel);
   ol.popGeneratorState();
   //2.}
 
@@ -224,53 +226,53 @@ void PageDefImpl::writeDocumentation(OutputList &ol)
   // save old generator state and write title only to Man generator
   ol.pushGeneratorState();
   //2.{
-  ol.disableAllBut(OutputGenerator::Man);
+  ol.disableAllBut(OutputType::Man);
   ol.startTitleHead(manPageName);
   ol.endTitleHead(manPageName, manPageName);
   if (si)
   {
-    ol.pushGeneratorState();
-    ol.disableAllBut(OutputGenerator::Man);
     ol.writeString(" - ");
-    ol.popGeneratorState();
 
     if (si->title() != manPageName)
     {
       ol.generateDoc(docFile(),getStartBodyLine(),this,0,si->title(),TRUE,FALSE,
-                     0,TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+                     QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
       ol.endSection(si->label(),si->type());
     }
   }
   ol.popGeneratorState();
   //2.}
 
-  // for Latex the section is already generated as a chapter in the index!
   ol.pushGeneratorState();
   //2.{
-  ol.disable(OutputGenerator::Latex);
-  ol.disable(OutputGenerator::Docbook);
-  ol.disable(OutputGenerator::RTF);
-  ol.disable(OutputGenerator::Man);
-  if (hasTitle() && !name().isEmpty() && si!=0)
+  ol.disable(OutputType::Man);
+  QCString title;
+  if (this == Doxygen::mainPage.get() && !hasTitle())
+    title = theTranslator->trMainPage();
+  else
+    title = m_title;
+
+  if (!title.isEmpty() && !name().isEmpty() && si!=0)
   {
     ol.startPageDoc(si->title());
     //ol.startSection(si->label,si->title,si->type);
-    startTitle(ol,getOutputFileBase(),this);
-    ol.generateDoc(docFile(),getStartBodyLine(),this,0,si->title(),TRUE,FALSE,
-                   0,TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
-    //stringToSearchIndex(getOutputFileBase(),
-    //                    theTranslator->trPage(TRUE,TRUE)+" "+si->title,
-    //                    si->title);
-    //ol.endSection(si->label,si->type);
-    endTitle(ol,getOutputFileBase(),name());
+    ol.startHeaderSection();
+    ol.startTitleHead(getOutputFileBase());
+    ol.parseText(title);
+    ol.endTitleHead(getOutputFileBase(),title);
+    ol.endHeaderSection();
+
+    /*ol.generateDoc(docFile(),getStartBodyLine(),this,0,si->title(),TRUE,FALSE,
+                   QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT));*/
   }
   else
     ol.startPageDoc("");
-  ol.startContents();
   ol.popGeneratorState();
   //2.}
 
-  if ((m_localToc.isHtmlEnabled() || m_localToc.isLatexEnabled() || m_localToc.isDocbookEnabled()) && hasSections())
+  ol.startContents();
+  if ((m_localToc.isHtmlEnabled() || m_localToc.isLatexEnabled() || m_localToc.isDocbookEnabled())
+    && hasSections())
   {
     writeToc(ol, m_localToc);
   }
@@ -281,7 +283,7 @@ void PageDefImpl::writeDocumentation(OutputList &ol)
 
   if (generateTreeView && getOuterScope()!=Doxygen::globalScope && !Config_getBool(DISABLE_INDEX))
   {
-    endFileWithNavPath(getOuterScope(),ol);
+    endFileWithNavPath(ol,getOuterScope());
   }
   else
   {
@@ -290,18 +292,16 @@ void PageDefImpl::writeDocumentation(OutputList &ol)
 
   ol.popGeneratorState();
   //1.}
-
-  Doxygen::indexList->addIndexItem(this,0,0,filterTitle(title()));
 }
 
-void PageDefImpl::writePageDocumentation(OutputList &ol)
+void PageDefImpl::writePageDocumentation(OutputList &ol) const
 {
   ol.startTextBlock();
-  QCString docStr = documentation()+inbodyDocumentation();
+  QCString docStr = (briefDescription().isEmpty()?"":briefDescription()+"\n\n")+documentation()+inbodyDocumentation();
   if (hasBriefDescription() && !SectionManager::instance().find(name()))
   {
     ol.pushGeneratorState();
-    ol.disableAllBut(OutputGenerator::Man);
+    ol.disableAllBut(OutputType::Man);
     ol.writeString(" - ");
     ol.popGeneratorState();
   }
@@ -313,7 +313,7 @@ void PageDefImpl::writePageDocumentation(OutputList &ol)
       docStr,              // docStr
       TRUE,                // index words
       FALSE,               // not an example
-      0,                   // exampleName
+      QCString(),                   // exampleName
       FALSE,               // singleLine
       FALSE,               // linkFromIndex
       TRUE                 // markdown support
@@ -326,31 +326,13 @@ void PageDefImpl::writePageDocumentation(OutputList &ol)
     // parent page.
     ol.pushGeneratorState();
     ol.disableAll();
-    ol.enable(OutputGenerator::Latex);
-    ol.enable(OutputGenerator::Docbook);
-    ol.enable(OutputGenerator::RTF);
+    ol.enable(OutputType::Latex);
+    ol.enable(OutputType::Docbook);
+    ol.enable(OutputType::RTF);
 
-    PageSDict::Iterator pdi(*m_subPageDict);
-    PageDef *subPage=pdi.toFirst();
-    for (pdi.toFirst();(subPage=pdi.current());++pdi)
+    for (const auto &subPage : m_subPages)
     {
-      SectionType sectionType = SectionType::Paragraph;
-      switch (m_nestingLevel)
-      {
-        case  0: sectionType = SectionType::Page;          break;
-        case  1: sectionType = SectionType::Section;       break;
-        case  2: sectionType = SectionType::Subsection;    break;
-        case  3: sectionType = SectionType::Subsubsection; break;
-        default: sectionType = SectionType::Paragraph;     break;
-      }
-      QCString title = subPage->title();
-      if (title.isEmpty()) title = subPage->name();
-      ol.startSection(subPage->name(),title,sectionType);
-      ol.parseText(title);
-      ol.endSection(subPage->name(),sectionType);
-      Doxygen::subpageNestingLevel++;
-      subPage->writePageDocumentation(ol);
-      Doxygen::subpageNestingLevel--;
+      ol.writePageLink(subPage->getOutputFileBase(), FALSE);
     }
 
     ol.popGeneratorState();
@@ -359,12 +341,11 @@ void PageDefImpl::writePageDocumentation(OutputList &ol)
 
 bool PageDefImpl::visibleInIndex() const
 {
-  static bool externalPages = Config_getBool(EXTERNAL_PAGES);
+  bool externalPages = Config_getBool(EXTERNAL_PAGES);
   return // not part of a group
          !getGroupDef() &&
          // not an externally defined page
-         (!isReference() || externalPages)
-         ;
+         (!isReference() || externalPages);
 }
 
 bool PageDefImpl::documentedPage() const
@@ -377,7 +358,7 @@ bool PageDefImpl::documentedPage() const
 
 bool PageDefImpl::hasSubPages() const
 {
-  return m_subPageDict->count()>0;
+  return !m_subPages.empty();
 }
 
 void PageDefImpl::setNestingLevel(int l)
@@ -405,6 +386,11 @@ bool PageDefImpl::hasTitle() const
   return !m_title.isEmpty() && m_title.lower()!="notitle";
 }
 
+void PageDefImpl::setTitle(const QCString &title)
+{
+  m_title = title;
+}
+
 // --- Cast functions
 
 PageDef *toPageDef(Definition *d)
